
/*
----------------------------------------------------------------------


Top16 - II  "ATc"    
Operating system


(c) TCTEC Pty Ltd  www.tctec.net


Settings:

MegaTinyCore
ATTiny806  
OR
ATTiny1606

no bootloader
Clock: 8MHz internal
millis/micros enabled
UPDI No reset pin
BOD voltage level 2.6
* Do not use IO 6 (PB3)



09-Dec-2022          RVW      Version ATc         Created


----------------------------------------------------------------------
*/

/* ============================================================================
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
===============================================================================
*/
#include <Wire.h>
#include "ADS1100.h"




#define LED (13)

// Address of ADS1100A3
#define ADC_ADRESS 0x4B

// Address of ADS110  default
// #define ADS_ADRESS 0x48

struct GLOBALS {

   int gainSetting ;
   int resolution ;
   
}globals;


ADS1100 adc=ADS1100(ADC_ADRESS);

int outputPinMap[8] = {0,1,2,3,4,5,16,15};

const int SERIALBUFSIZE = 16 ;
char  rxBuffer[SERIALBUFSIZE] ;
int rxBufCount = 0 ;
int analogReadings[8] ;

void setupDigitalOutputs()
{
  int count = 0 ;
  
  for(count = 0 ; count < 8; count++)
  {
    pinMode(outputPinMap[count], OUTPUT) ;
  }
}

void processSerial()
{
  int in ;
  in = Serial.read() ;
  
  if (in > -1)
  {        
    // command ?
    //
    if ((in == '\r') || (in == '\n'))
    {
      processCommand(rxBuffer) ;
      
      rxBuffer[rxBufCount] = 0 ;
           
      // reset the buffer
      //
      rxBufCount = 0 ;
      
    }    
    else
    {
      rxBuffer[rxBufCount] = in ;     
      rxBufCount++ ;
    
      if (rxBufCount > SERIALBUFSIZE)
      {
        rxBufCount = 0 ;
      }
    }
  }  
}

void setup() {   


    setupDigitalOutputs() ;
   
    setupTimer();  

    // analog inputs with ADS1100
    //
    adc.begin();
    

    // start up the AtoD converter
    //
    globals.resolution = 12;
    setGain(1);
        

    // MUX for anlaog inputs
    //
    PORTC.DIRSET = 0b00000111;
    PORTA.DIRSET = 0b00000111;
   
     
    
    // LED
    //
    pinMode(LED, OUTPUT) ;
    digitalWrite(LED, HIGH) ;
    
    digitalWrite(A0, LOW) ;   
    digitalWrite(A2, LOW) ;
          
    Wire.begin();  // join I2C bus
    Serial.begin(115200); // initialize serial communication 
   
 
}

double volts ;

void callback()
{
  
}

void loop() 
  {
    static byte outputState = 0x01 ;
    static byte outputCount = 0 ;
    static long loopCount = 0 ;

    float v = 0.0;

    
    loopCount++ ;
    
    if (loopCount > 20000)  // ~ 100 mSec
    {
      loopCount = 0 ; 
      digitalWrite(LED, HIGH) ;     
    }
    
    //  setDigitalOutputs(outputState, 0xFF) ;
         
    outputState <<= 1;
    
    outputCount++ ;
    
    if (outputCount >= 8)
    {
      outputCount = 0 ; 
      outputState = 0x01 ;    
    }
       
    
    processSerial();      
  }

  
  
void setDigitalOutputs(byte state, byte mask)
{
  
  int count = 0 ;
  int checkMask = 0x01 ;
  
  // if the mask bit is set, then change the state of the output
  //
  
  for(count = 0 ; count < 8; count++)
  {
    if (checkMask & mask)
    {
      // disable PWM
      //
      disablePWM(count);
      
      if (checkMask & state)
      {
        digitalWrite(outputPinMap[count], HIGH);
      }
      else
      {
        digitalWrite(outputPinMap[count], LOW) ;
      }
    }
    checkMask <<= 1 ;    
  }   
}

// 
//
void setGain(uint8_t gain)
{    
    
    globals.gainSetting = gain ;  

    uint8_t datarate = ADS1100_DATA_RATE_128SPS;
    uint8_t newgain =  ADS1100_GAIN_1X;

    if (gain <=0)
    {
      return;
    }
    if (gain > 4)
    {
      return;
    }

    switch(globals.resolution)
    {
      case 12:
      datarate = ADS1100_DATA_RATE_128SPS;
      break;

      case 14:
      datarate = ADS1100_DATA_RATE_32SPS;
      break;

      case 15:
      datarate = ADS1100_DATA_RATE_16SPS;
      break;

      case 16:
      datarate = ADS1100_DATA_RATE_8SPS;
      break;
    }
    
    switch(globals.gainSetting)
    {
      case 1:
      newgain =  ADS1100_GAIN_1X;      
      break;
      
      case 2:
      newgain =  ADS1100_GAIN_2X;    
      break;

      case 3:
      newgain =  ADS1100_GAIN_4X;    
      break;

      case 4:
      newgain =  ADS1100_GAIN_8X;    
      break;
                  
    }
      
      adc.configure(ADS1100_CONVERSION_SINGLE,datarate, newgain);
}

//--------------------------------------------------------------------------------------------------
// read ADS1100 mv voltage using reference voltage
//
// resolution = 12 for fast read
//              16 for slow, more precise read. 
//
// input = 0 to 7
//--------------------------------------------------------------------------------------------------

double ads1100getVoltage(int resolution, int input)
{
 
 int16_t ref_reading = 0;
 int16_t  input_reading = 0;
 float gain = 1.0; 
 double calibrated_voltage = 0.0;
   
 int timeout = 0;

 // default power-on is 8SPS, 16 bit resolution, only change the resolution if necessary
 // Typically, we will do multiple readings at a particular resolution
 //
 if(resolution == 16)
 {
    if (globals.resolution != 16)
    {
      adc.configure(ADS1100_CONVERSION_SINGLE,ADS1100_DATA_RATE_8SPS,ADS1100_GAIN_1X);
    }
 }
 else if(resolution == 12)
 {
    if(globals.resolution != 12)
    {
      adc.configure(ADS1100_CONVERSION_SINGLE,ADS1100_DATA_RATE_128SPS,ADS1100_GAIN_1X);
    }
  
 }
 else if(resolution == 14)
 {
  if(globals.resolution != 14)
    {
      adc.configure(ADS1100_CONVERSION_SINGLE,ADS1100_DATA_RATE_32SPS,ADS1100_GAIN_1X);  
    }
 }
 else
 {
   // invalid resolution
   //
   return(0);  
 }

 // remember the setting, so only re-send setup command to ADS1100 if necessary
 //
 globals.resolution = resolution;

 // get reference reading
 //
 setanalogMUX(8);   
 
 adc.startSingleConversion(); // the function starts the conversion process.
 while (!adc.conversionDone())  // the function reads the value and the status. 
 {
   delay(2);
   if (++timeout > 100)
   {
      break;
    }    
 }  
 timeout = 0;
 ref_reading = adc.getvalue();  
 
 // get input reading
 
 setanalogMUX(input);
 delay(2); 
 adc.startSingleConversion(); // the function starts the conversion process.
 while (!adc.conversionDone())  // the function reads the value and the status. 
 {
   delay(2);
   if (++timeout > 100)
   {
      break;
    }    
 }   
 timeout = 0; 
 input_reading = adc.getvalue();

 if (globals.gainSetting == 3)
 {
  gain = 4.0;  
 }
 else if(globals.gainSetting == 4)
 {
  gain = 8.0;
 }
 else
 {
   gain = globals.gainSetting;
 }


 calibrated_voltage = 2.5 / ref_reading;
 calibrated_voltage *= input_reading;
 calibrated_voltage *= gain;
 return(calibrated_voltage);  
   
}

/*
 * Not Used
Continuously read the inputs and update the reading array 
*/
void readNextAnalogInput()
{
  static byte input = 0 ;  

  return;
  
  // get sample
  //
  //analogReadings[input] = adc0.getConversionP0GND();
  
  input++ ;  
  if (input > 7)
  {
    input = 0 ;
  }
  // set MUX address for next input
  //  
  PORTC.OUTCLR =(0b00000111);
  PORTC.OUTSET = (input) ;
  
   
}

// input '8' is the reference voltage
//
void setanalogMUX(int input)
{
  int actualInput = 0 ;
  
  switch (input)
  {
    case 7: actualInput = 0 ; break ;
    case 6: actualInput = 1 ; break ;
    case 5: actualInput = 2 ; break ;
    case 4: actualInput = 3 ; break ;
    case 3: actualInput = 4 ; break ;
    case 2: actualInput = 5 ; break ;
    case 1: actualInput = 6 ; break ;
    case 0: actualInput = 7 ; break ;
    
  }

  if(input == 8)
  {
    // reference voltage reading
    //
    // set MUX address 
    //    
    PORTC.OUTCLR =(0b00000111);
    PORTC.OUTSET =(0b00000000);  
    PORTA.OUTSET =(0b00000010);
  }
  else
  {
    // Reading and input  
    // set MUX address 
    //    
    PORTA.OUTCLR =(0b00000010);   
    PORTC.OUTCLR =(0b00000111);
    PORTC.OUTSET = (actualInput) ;
  }
    
  delay(5) ;     
  
}


  

double readAnalogIn(int input)
{ 
  int actualInput = 0 ;
  double v = 0.0;
  
  if (input > 8)
  {
    input = 1 ;
  }
  input-- ;
  
  switch (input)
  {
    case 7: actualInput = 0 ; break ;
    case 6: actualInput = 1 ; break ;
    case 5: actualInput = 2 ; break ;
    case 4: actualInput = 3 ; break ;
    case 3: actualInput = 4 ; break ;
    case 2: actualInput = 5 ; break ;
    case 1: actualInput = 6 ; break ;
    case 0: actualInput = 7 ; break ;
    
  }
     

  v = ads1100getVoltage(12, input);
  
  // convert to milli volts
  //
  v *= 1000.0;
  
  return(v); 
   

}

int readAllDigitalInputs()
{
  int digitalState = 0 ;
  int count = 8 ;
    
  
  // read all the digital inputs
  //
  
  for(count = 0; count < 7; count++)
  {
    // read analog voltages at 12 bit resolution and convert to digital
    //
    if(ads1100getVoltage(12, count) > 2.5)   // 2.5 volts
    {
      digitalState |= 1 ;
    }
    else
    {      
    }
    digitalState <<= 1;
  }
  
  digitalState >>= 1 ;
  
  return(digitalState);  
  
}

int readDigitalInput(int input)
{  
  
  int digitalState = 0 ; 
  int i = input - 1;

   if(i < 0)
   {
    return 0;
   }
   
  // read analog voltages and convert to digital
  //
  if(ads1100getVoltage(12, i) > 2.5)   // 2.5 volts
  {
    digitalState = 1 ;
  }
  else
  {
    digitalState = 0 ;
  }
   
  return(digitalState) ;  
  
}


/*

PWM output

*/
void SETPWM(int pin, int Value)
{
  byte newValue = Value;

  newValue >>= 1;

  setPWM(pin-1, newValue);
 
  
}
